import json
import requests
import lxml.etree as ET

from django.conf import settings

base_url = "https://transkribus.eu/TrpServer/rest"
nsmap = {
    "page": "http://schema.primaresearch.org/PAGE/gts/pagecontent/2013-07-15"
}
crowd_base_url = "https://transkribus.eu/r/read/sandbox/application/?colId={}&docId={}&pageId={}"

try:
    user = settings.TRANSKRIBUS['user']
except (AttributeError, KeyError) as e:
    print("no TRANSKRIBUS user set in the project's settings file")
    print(e)
    user = "user"

try:
    pw = settings.TRANSKRIBUS['pw']
except (AttributeError, KeyError) as e:
    print("no TRANSKRIBUS pw set in the project's settings file")
    print(e)
    pw = "pw"

try:
    col_id = settings.TRANSKRIBUS['col_id']
except (AttributeError, KeyError) as e:
    print("no TRANSKRIBUS col_id set in the project's settings file")
    print(e)
    col_id = None


def trp_login(user, pw, base_url=base_url):
    """ log in function
        :param user: Your TRANSKRIBUS user name, e.g. my.mail@whatever.com
        :param pw: Your TRANSKRIBUS password
        :param base_url: The base URL of the TRANSKRIBUS API
        :return: The Session ID in case of a successful log in attempt
    """
    url = "{}/auth/login".format(base_url)
    payload = "user={}&pw={}".format(user, pw)
    headers = {
        'Content-Type': "application/x-www-form-urlencoded",
        }
    response = requests.request("POST", url, data=payload, headers=headers)
    if response.ok:
        session_id = response.text.split('<sessionId>')[1].split('</sessionId>')[0]
        return session_id
    else:
        response.ok


def trp_ft_search(base_url=base_url, user=user, pw=pw, **kwargs):
    """ Helper function to interact with TRANSKRIBUS fulltext search endpoint
        :param user: Your TRANSKRIBUS user name, e.g. my.mail@whatever.com
        :param pw: Your TRANSKRIBUS password
        :param base_url: The base URL of the TRANSKRIBUS API
        :param kwargs: kwargs will be forwarded to TRANSKRIBUS API endpoint e.g. 'query' holds the\
        search string
        :return: The default TRANSKRIBUS response as JSON
    """
    url = f"{base_url}/search/fulltext"
    if kwargs:
        querystring = kwargs
    else:
        return False
    querystring['type'] = "LinesLc"
    print(querystring)
    session_id = trp_login(user, pw, base_url=base_url)
    headers = {
        'cookie': "JSESSIONID={}".format(session_id),
    }
    response = requests.request("GET", url, headers=headers, params=querystring)
    if response.ok:
        return response.json()
    else:
        return response.ok


def trp_get_doc_md(doc_id, base_url=base_url, col_id=col_id, user=user, pw=pw):
    """ Helper function to interact with TRANSKRIBUS document metadata endpoint
        :param user: Your TRANSKRIBUS user name, e.g. my.mail@whatever.com
        :param pw: Your TRANSKRIBUS password
        :param base_url: The base URL of the TRANSKRIBUS API
        :col_id: The ID of a TRANSKRIBUS Collection
        :doc_id: The ID of TRANSKRIBUS Document
        :page_id: The page number of the Document
        :return: A dict with basic metadata of a transkribus Document
    """
    url = f"{base_url}/collections/{col_id}/{doc_id}/metadata"
    session_id = trp_login(user, pw, base_url=base_url)
    headers = {
        'cookie': "JSESSIONID={}".format(session_id),
    }
    response = requests.request("GET", url, headers=headers)
    return response.json()


def trp_get_doc_overview_md(doc_id, base_url=base_url, col_id=col_id, user=user, pw=pw):
    """ Helper function to interact with TRANSKRIBUS document endpoint
        :param user: Your TRANSKRIBUS user name, e.g. my.mail@whatever.com
        :param pw: Your TRANSKRIBUS password
        :param base_url: The base URL of the TRANSKRIBUS API
        :col_id: The ID of a TRANSKRIBUS Collection
        :doc_id: The ID of TRANSKRIBUS Document
        :return: A dict with basic metadata of a transkribus Document
    """
    url = f"{base_url}/collections/{col_id}/{doc_id}/fulldoc"
    session_id = trp_login(user, pw, base_url=base_url)
    headers = {
        'cookie': "JSESSIONID={}".format(session_id),
    }
    response = requests.request("GET", url, headers=headers)
    if response.ok:
        result = {}
        result["trp_return"] = response.json()
        page_list = result["trp_return"]["pageList"]["pages"]
        result["pages"] = [
            {
                "page_id": x['pageId'],
                "doc_id": x['docId'],
                "page_nr": x['pageNr'],
                "thumb": x['thumbUrl']
            } for x in page_list
        ]
        return result
    else:
        return response.ok


def trp_get_fulldoc_md(doc_id, base_url=base_url, col_id=col_id, page_id="1", user=user, pw=pw):
    """ Helper function to interact with TRANSKRIBUS document endpoint
        :param user: Your TRANSKRIBUS user name, e.g. my.mail@whatever.com
        :param pw: Your TRANSKRIBUS password
        :param base_url: The base URL of the TRANSKRIBUS API
        :col_id: The ID of a TRANSKRIBUS Collection
        :doc_id: The ID of TRANSKRIBUS Document
        :page_id: The page number of the Document
        :return: A dict with basic metadata of a transkribus Document
    """
    url = f"{base_url}/collections/{col_id}/{doc_id}/{page_id}"
    session_id = trp_login(user, pw, base_url=base_url)
    headers = {
        'cookie': "JSESSIONID={}".format(session_id),
    }
    response = requests.request("GET", url, headers=headers)
    if response.ok:
        doc_xml = ET.fromstring(response.text.encode('utf8'))
        result = {
            "doc_id": doc_id,
            "base_url": base_url,
            "col_id": col_id,
            "page_id": page_id,
            "session_id": session_id
        }
        result["doc_url"] = url
        result["doc_xml"] = doc_xml
        result["transcript_url"] = doc_xml.xpath('//tsList/transcripts[1]/url/text()')[0]
        result["thumb_url"] = doc_xml.xpath('./thumbUrl/text()')[0]
        result["img_url"] = doc_xml.xpath('./url/text()')[0]
        result["img_url"] = doc_xml.xpath('./url/text()')[0]
        result["extra_info"] = trp_get_doc_md(
            doc_id, base_url=base_url, col_id=col_id, user=user, pw=pw
        )
        return result
    else:
        return response.ok


def get_transcript(fulldoc_md):
    """ Helper function to fetch the (latest) fulltext of a TRANSKRIBUS page
        :param fulldoc_md: A dict returned by trp_login.trp_get_fulldoc_md
        :return: The fulldoc_md dict with additional keys 'page_xml' and 'transcript'
    """
    nsmap = {
        "page": "http://schema.primaresearch.org/PAGE/gts/pagecontent/2013-07-15"
    }
    md = fulldoc_md
    url = md['transcript_url']
    headers = {
        'cookie': "JSESSIONID={}".format(md['session_id']),
    }
    response = requests.request("GET", url, headers=headers)
    if response.ok:
        page = ET.fromstring(response.text.encode('utf8'))
        md['page_xml'] = page
        md['transcript'] = page.xpath(
            './/page:TextLine//page:Unicode/text()', namespaces=nsmap
        )
        return md
    else:
        return response.ok


def list_documents(base_url=base_url, col_id=col_id, user=user, pw=pw):
    """ Helper function to interact with TRANSKRIBUS collection endpoint to list all documents
        :param user: Your TRANSKRIBUS user name, e.g. my.mail@whatever.com
        :param pw: Your TRANSKRIBUS password
        :param base_url: The base URL of the TRANSKRIBUS API
        :col_id: The ID of a TRANSKRIBUS Collection
        :return: A dict with the default TRANSKRIBUS API return
    """
    url = f"{base_url}/collections/{col_id}/list"
    session_id = trp_login(user, pw, base_url=base_url)
    headers = {
        'cookie': "JSESSIONID={}".format(session_id),
    }
    response = requests.request("GET", url, headers=headers)
    if response.ok:
        return response.json()
    else:
        return response.ok



from django.conf.urls import url
from django.urls import path
from django.views.generic.base import RedirectView
from . import views

app_name = 'transkribus'


urlpatterns = [
    url(r'^search', views.TrpSearchView.as_view(), name="trp_search"),
    url(r'^hits', views.TrpSearchResultView.as_view(), name="trp_hits"),
    url(r'^docs', views.TrpListView.as_view(), name="trp_docs"),
    path('document/<col_id>/<doc_id>', views.TrpDocumentView.as_view(), name='trp_document'),
    path('page/<col_id>/<doc_id>/<page_id>', views.TrpPageView.as_view(), name='trp_page'),
]



from django.conf import settings

TRANSKRIBUS_TRANSLATIONS_DEFAULT = {
    'search_form': {
        'prefil': 'search in all documents',
        'button': 'Go!'
    },
    'search_header': {
        'header': 'Fulltext Search'
    },
    'hits': {
        'facet_header': 'refine your search',
        'facet_doc_title': 'Document',
        'result_header': 'Results',
        'kwic_header': 'KWIC',
        'result_col': 'Collection',
        'result_doc': 'Document',
        'result_page': 'Page',
        'result_link': 'go to document',
        'hits': 'hits',
        'prev': 'prev',
        'next': 'next'
    },
    'page': {
        'img_col': 'IMG',
        'text_col': 'TEXT'
    },
    'docs': {
        'title_col': 'Title',
        'page_nr_col': 'Nr. of pages',
        'preview_col': 'Preview',
        'doc_singular': 'Document',
        'doc_plural': 'Documents',
        'page_singular': 'Page',
        'page_plural': 'Pages'
    }
}


try:
    TRANSKRIBUS_TRANSLATIONS = settings.TRANSKRIBUS_TRANSLATIONS
except AttributeError:
    TRANSKRIBUS_TRANSLATIONS = TRANSKRIBUS_TRANSLATIONS_DEFAULT



from django.db import models
from django.urls import reverse

from transkribus.trp_utils import (
    trp_get_doc_overview_md,
    crowd_base_url,
    list_documents,
    col_id
)


class TrpBaseModel(models.Model):
    """ An abstract base class to add transkribus collection and document IDs to an object """
    col_id = models.CharField(
        max_length=25, blank=True, verbose_name="Collection ID",
        help_text="The collection's Identifier"
    )
    doc_id = models.CharField(
        max_length=25, blank=True, verbose_name="Document ID",
        help_text="The document's Identifier"
    )

    class Meta:
        abstract = True

    def trp_fetch_md(self):
        try:
            result = trp_get_doc_overview_md(self.doc_id)
        except Exception as e:
            result = None
            print(e)
        return result

    def trp_fetch_crowd_link(self):
        try:
            result = crowd_base_url.format(self.col_id, self.doc_id, '1')
        except Exception as e:
            print(e)
            result = None
        return result

    def trp_fetch_documents(self):
        try:
            result = list_documents()
        except Exception as e:
            print(e)
            result = None
        return result

    @classmethod
    def trp_get_docs_url(self):
        return reverse('transkribus:trp_docs')

    def trp_get_document_url(self):
        return reverse(
            'transkribus:trp_docs',
            kwargs={
                'col_id': self.col_id,
                'doc_id': self.doc_id
            }
        )

    def trp_get_document_url(self):
        return reverse(
            'transkribus:trp_docs',
            kwargs={
                'col_id': self.col_id,
                'doc_id': self.doc_id
            }
        )

    def trp_get_page_url(self):
        return reverse(
            'transkribus:trp_docs',
            kwargs={
                'col_id': self.col_id,
                'doc_id': self.doc_id,
                'page_id': self.page_id
            }
        )



from django.conf import settings
from django.contrib.auth.mixins import UserPassesTestMixin
from django.conf import settings
from django.shortcuts import render
from django.urls import reverse
from django.views.generic import TemplateView

from transkribus.trp_utils import (
    user,
    pw,
    col_id,
    base_url,
    trp_login,
    trp_ft_search,
    trp_get_fulldoc_md,
    get_transcript,
    list_documents,
    trp_get_doc_overview_md,
    crowd_base_url
)

TRP_PUBLIC = getattr(settings, 'TRANSKRIBUS_PUBLIC', False)

try:
    APIS_OSD_JS = settings.APIS_OSD_JS
except AttributeError:
    APIS_OSD_JS = (
        "https://cdnjs.cloudflare.com/ajax/libs/openseadragon/2.4.0/openseadragon.min.js"
    )
try:
    APIS_OSD_IMG_PREFIX = settings.APIS_OSD_IMG_PREFIX
except AttributeError:
    APIS_OSD_IMG_PREFIX = (
        "https://cdnjs.cloudflare.com/ajax/libs/openseadragon/2.4.0/images/"
    )


class TrpSearchView(TemplateView):
    template_name = 'transkribus/search.html'


class TrpSearchResultView(TemplateView):
    template_name = 'transkribus/hits.html'

    def get_context_data(self, **kwargs):
        additional_filters = [f"collectionId:{col_id}", ]
        context = super().get_context_data(**kwargs)
        if self.request.GET.get('filter'):
            additional_filters.append(self.request.GET.get('filter'))
        print(f"additional_filters: {additional_filters}")
        query = self.request.GET.get('query')
        kwargs = {
            'query': query,
            'filter': set(additional_filters),
            'start': self.request.GET.get('start', '0'),
            'rows': self.request.GET.get('rows', '25')
        }
        filterstring = "&filter=".join(additional_filters)
        try:
            result = trp_ft_search(
                base_url, user, pw, **kwargs
            )
        except Exception as e:
            context['trp_fetch_error'] = e
            print(e)
            result = None
        if result:
            context['trp_result'] = result
            context['hits'] = result['numResults']
            context['rows'] = kwargs['rows']
            context['start'] = kwargs['start']
            context['base_url'] = f"{self.request.path}?query={query}"
            if "f_title" in filterstring:
                context['clear_filter'] = True
            context['new_url'] = f"{self.request.path}?query={query}&filter={filterstring}"
            if int(context['rows']) + int(context['start']) < int(context['hits']):
                context['next'] = int(context['rows']) + int(context['start'])
            prev = int(context['start']) - int(context['rows'])
            if prev >= 0:
                context['prev'] = prev
            else:
                context['prev'] = 0
            return context


class TrpListView(UserPassesTestMixin, TemplateView):
    template_name = 'transkribus/documents.html'

    def test_func(self):
        if self.request.user.is_authenticated or TRP_PUBLIC:
            return True
        else:
            return False

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        result = list_documents(base_url=base_url, col_id=col_id, user=user, pw=pw)
        try:
            context['collection'] = result[0]['collectionList']['colList'][0]
        except IndexError:
            context['collection'] = None
        context['result'] = result
        context['nr_docs'] = len(result)
        context['col_id'] = col_id
        return context


class TrpDocumentView(UserPassesTestMixin, TemplateView):
    template_name = 'transkribus/doc_overview.html'

    def test_func(self):
        if self.request.user.is_authenticated or TRP_PUBLIC:
            return True
        else:
            return False

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['doc_md'] = trp_get_doc_overview_md(
            base_url=base_url, user=user, pw=pw, **self.kwargs
        )
        context['col_id'] = self.kwargs['col_id']
        context['doc_id'] = self.kwargs['doc_id']
        context['edit_link'] = crowd_base_url.format(
            self.kwargs['col_id'],
            self.kwargs['doc_id'],
            '1'
        )
        return context


class TrpPageView(UserPassesTestMixin, TemplateView):
    template_name = 'transkribus/page.html'

    def test_func(self):
        if self.request.user.is_authenticated or TRP_PUBLIC:
            return True
        else:
            return False

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        fulldoc_md = trp_get_fulldoc_md(
            base_url=base_url, user=user, pw=pw, **self.kwargs
        )
        result = get_transcript(fulldoc_md)
        page_id = int(self.kwargs['page_id'])
        context['result'] = result
        context['first_page'] = reverse('transkribus:trp_page', kwargs={
            'col_id': result['col_id'],
            'doc_id': result['doc_id'],
            'page_id': '1'
        })
        context['last_page'] = reverse('transkribus:trp_page', kwargs={
            'col_id': result['col_id'],
            'doc_id': result['doc_id'],
            'page_id': result['extra_info']['nrOfPages']
        })
        if page_id > 1:
            context['prev'] = reverse('transkribus:trp_page', kwargs={
                'col_id': result['col_id'],
                'doc_id': result['doc_id'],
                'page_id': f"{page_id-1}"
            })
        else:
            context['prev'] = None

        if page_id < int(result['extra_info']['nrOfPages']):
            context['next'] = reverse('transkribus:trp_page', kwargs={
                'col_id': result['col_id'],
                'doc_id': result['doc_id'],
                'page_id': f"{page_id+1}"
            })
        else:
            context['next'] = None

        context['openseadragon_js'] = APIS_OSD_JS
        context['openseadragon_img'] = APIS_OSD_IMG_PREFIX
        context['edit_link'] = crowd_base_url.format(
            self.kwargs['col_id'],
            self.kwargs['doc_id'],
            page_id
        )
        return context
